#!/usr/bin/python3.5
#Version 23
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning

import xml.etree.ElementTree as ET
import sys
import shutil
import argparse
from datetime import date
import datetime
import json

parser = argparse.ArgumentParser()
parser.add_argument("--hmc", help="Specify the HMC IP Address or Hostname")
parser.add_argument("--username", help="username for the HMC")
parser.add_argument("--password", help="password for the HMC")
parser.add_argument("--path", help="Output directory where the responses and CSV data is saved")
parser.add_argument("--type", help="Types: RawMetrics | ProcessedMetrics(default) | AggregatedMetrics")
parser.add_argument("--ssp", help="SSP name or ALL(default)")
parser.add_argument("--debug", help="Output details as we go",action="store_true")
parser.add_argument("--save", help="Save the HMC responses in files for Diags & Learning",action="store_true")
parser.add_argument("--show", help="List the data on the screen",action="store_true")
parser.add_argument("--perfs", help="Updated the SSP preferences",action="store_true")
parser.add_argument("--vios", help="Add VIOS KB/s stats",action="store_true")
parser.add_argument("--googlechart", help="Save data in Googlechart ready format",action="store_true")

args = parser.parse_args()
hostName = args.hmc
userName = args.username
password = args.password
path = args.path
getType = args.type
sspNeeded = args.ssp
debug = args.debug
save = args.save
show = args.show
perfs = args.perfs
vios = args.vios
googlechart = args.googlechart

if(userName == None):
        sys.exit('HMC --username  is mandatory')
if(password == None):
        sys.exit('HMC --password  is mandatory')
if(hostName == None):
        sys.exit('HMC --hmc <IP or host name>  is mandatory')
if(path == None):
        sys.exit('Output --path (directory> archive is mandatory')
if(sspNeeded == None):
        sspNeeded = "ALL"
if(getType == None):
        getType = "ProcessedMetrics"  # this is the default
if (getType != "RawMetrics" and getType != "ProcessedMetrics" and getType != "AggregatedMetrics"):
        sys.exit('Output --type RawMetrics|ProcessedMetrics|AggregatedMetrics Mandatory one of these')
if(perfs == None):
        perfs = 0
#if(debug == None):
#        debug = 0
#if(save == None):
#        save = 0
#if(show == None):
#        show = 0

print("Switching off ugly Security Warnings 'Unverified HTTPS request is being made'")
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
print("------- SSP: %s   Data type = %s   Set Preferences:%d (0=No)" %(sspNeeded,getType,perfs))

##############################################
# _                            
#| |    ___   __ _  ___  _ __  
#| |   / _ \ / _` |/ _ \| '_ \ 
#| |__| (_) | (_| | (_) | | | |
#|_____\___/ \__, |\___/|_| |_|
#            |___/             
logonheaders = {'Content-Type': 'application/vnd.ibm.powervm.web+xml; type=LogonRequest'}
logonUrl =  'https://'+hostName+':12443/rest/api/web/Logon'
logonPayload = '<LogonRequest schemaVersion="V1_0" xmlns="http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/" xmlns:mc="http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/"><UserID>'+userName+'</UserID><Password>'+ password +'</Password></LogonRequest>'

r = requests.put(logonUrl,data=logonPayload,headers=logonheaders,verify=False)
if r.status_code != 200:
        print("Error: Logon failed error code=%d url=%s" %(r.status_code,logonUrl))
        if debug:
                print("Returned:" + r.text)
        sys.exit(r.status_code)

#Extract from the XML the token
xmlResponse =  r.text
xmlResponse = ET.fromstring(xmlResponse)
sessionToken = xmlResponse[1].text
if debug:
        print("Log on response=%d and got Token=\n----\n%s\n----" %(r.status_code, sessionToken))

#################################################
#            _     ____ ____  ____  
#  __ _  ___| |_  / ___/ ___||  _ \ 
# / _` |/ _ \ __| \___ \___ \| |_) |
#| (_| |  __/ |_   ___) |__) |  __/ 
# \__, |\___|\__| |____/____/|_|    
# |___/                             
#                 __                                   
# _ __  _ __ ___ / _| ___ _ __ ___ _ __   ___ ___  ___ 
#| '_ \| '__/ _ \ |_ / _ \ '__/ _ \ '_ \ / __/ _ \/ __|
#| |_) | | |  __/  _|  __/ | |  __/ | | | (_|  __/\__ \
#| .__/|_|  \___|_|  \___|_|  \___|_| |_|\___\___||___/
#|_|                                                   

print("------- Get Preferences")
prefHeaders = {'X-API-Session' : sessionToken}
prefUrl = 'https://'+hostName+':12443/rest/api/pcm/preferences/SSP'
r = requests.get(prefUrl, headers=prefHeaders,verify=False)
if r.status_code != 200:
        print("Error: GET preferences error code=%d url=%s" %( r.status_code,prefUrl))
        if debug:
                print("Returned:" + r.text)
        sys.exit(r.status_code)

if save:
        date = date.today() 
        time =  datetime.datetime.now().time() 
        file_name = path + "/SSP-prefs.xml"
        file_object = open(file_name,"w")
        file_object.write(r.text)
        file_object.close()
        if debug:
            print("------- Successfully archived SSP Preferences to " + file_name)

# Convert the response TEXT to XML and saved in xmlResponse for later use
#    So we can extract the Management Server UUID
xmlResponse = r.text
xmlResponse = ET.fromstring(xmlResponse)

if (perfs):
    #                 _       _                         __
    # _   _ _ __   __| | __ _| |_ ___   _ __  _ __ ___ / _|___
    #| | | | '_ \ / _` |/ _` | __/ _ \ | '_ \| '__/ _ \ |_/ __|
    #| |_| | |_) | (_| | (_| | ||  __/ | |_) | | |  __/  _\__ \
    # \__,_| .__/ \__,_|\__,_|\__\___| | .__/|_|  \___|_| |___/
    #      |_|                         |_|
    print("------- Update Preferences")
    #Note that currently the prefereences have AggregationEnabled=true which forces two flags from false to true
    postBody = r.text
    # Set the flags to true for each SSP
    # Note: kb= and kxe= are in different orders!! But we can strip them out
    postBody = postBody.replace(' kb="UOD"',   '')
    postBody = postBody.replace(' kb="ROO"',   '')
    postBody = postBody.replace(' kb="ROR"',   '')
    postBody = postBody.replace(' kxe="false"','')
    postBody = postBody.replace(' kxe="true"', '')
    # Set AggregationEnabled=true & MonitorEnabled=true
    # Not clear if the Aggregate flag is needed for Processed
    postBody = postBody.replace('<AggregationEnabled>false</AggregationEnabled>',\
                                '<AggregationEnabled>true</AggregationEnabled>')
    postBody = postBody.replace('<MonitorEnabled>false</MonitorEnabled>',\
                                '<MonitorEnabled>true</MonitorEnabled>')

    # rip off the pre-amble
    i = postBody.find("<ManagementConsolePCMSSPPreference")
    postBody = postBody[i : ]

    # rip off the trailer
    i = postBody.find("</content>")
    postBody = postBody[ :i ]

    if save:
        file_name = path + "/SSP-prefs-updated.xml"
        file_object = open(file_name,"w")
        file_object.write(postBody)
        file_object.close()
        if debug:
            print("Save updated preferences to " + file_name)

    postHeaders = {'Content-Type': 'application/xml', 'X-API-Session' : sessionToken }
    postUrl = 'https://'+hostName+':12443/rest/api/pcm/preferences/SSP'
    r = requests.post(postUrl, headers=postHeaders, data=postBody, verify=False)
    if r.status_code != 200:
        print("Error: post preferences error code=%d url=%s" %( r.status_code,postUrl))
        if debug:
                print("URL::" + postUrl)
                print("Body:" + postBody)
                print("Returned:" + r.text)
        sys.exit(r.status_code)
    print("------- Successfully updated Preferences")
else:
    print("------- Not Updating Preferences")

# _____ _           _           ____ ____  ____  
#|  ___(_)_ __   __| |   __ _  / ___/ ___||  _ \ 
#| |_  | | '_ \ / _` |  / _` | \___ \___ \| |_) |
#|  _| | | | | | (_| | | (_| |  ___) |__) |  __/ 
#|_|   |_|_| |_|\__,_|  \__,_| |____/____/|_|    
                                                
print("Looking for Shared Storage Pool: %s" %(sspNeeded))
print("------- Find SSP UUIDs")
SSPName = 'notset'
for feed in xmlResponse:
    if 'entry' in feed.tag:
        for entry in feed:
            if 'content' in entry.tag:
                for content in entry:
                  if 'ManagementConsolePCMSSPPreference' in content.tag:
                    for pcm in content:
                        if 'ManagementConsoleSSPPreference' in pcm.tag:
                            for ssp in pcm:
                                if 'ClusterName' in ssp.tag:
                                        ClusterName = ssp.text
                                if 'ClusterUdId' in ssp.tag:
                                        ClusterUdId = ssp.text
                                if 'SSPName' in ssp.tag:
                                        SSPName = ssp.text
                                if 'SSPUuId' in ssp.tag:
                                        SSPUuId = ssp.text
                                if 'AggregationEnabled' in ssp.tag:
                                        AggregationEnabled = ssp.text
                                if 'MonitorEnabled' in ssp.tag:
                                        MonitorEnabled = ssp.text
                            if SSPName != 'notset': 
                                print("SSP=%s ClusterName=%s\n\tSSPUuId=%s\n\tAggregationEnabled=%s MonitorEnabled=%s" % \
                                        (SSPName,ClusterName,SSPUuId,AggregationEnabled,MonitorEnabled))
                            if sspNeeded == SSPName or sspNeeded == "ALL":
                                #################################################
                                #            _      __ _ _
                                #  __ _  ___| |_   / _(_) | ___ _ __   __ _ _ __ ___   ___  ___
                                # / _` |/ _ \ __| | |_| | |/ _ \ '_ \ / _` | '_ ` _ \ / _ \/ __|
                                #| (_| |  __/ |_  |  _| | |  __/ | | | (_| | | | | | |  __/\__ \
                                # \__, |\___|\__| |_| |_|_|\___|_| |_|\__,_|_| |_| |_|\___||___/
                                # |___/
        
                                                        
                                fileHeaders = {'X-API-Session' : sessionToken, 'Accept': 'application/atom+xml' }
                                fileUrl =  'https://'+hostName+':12443/rest/api/pcm/SharedStoragePool/'+SSPUuId+'/' + getType 

                                print("------- %s Stats for SSP:%s Cluster:%s" % (getType,SSPName,ClusterName))
                                r = requests.get(fileUrl,headers=fileHeaders,verify=False)
                                if r.status_code != 200:
                                        print("Error %d: Get SSP %s %s file of filenames \n\tURL=%s" \
                                              %( r.status_code,SSPName,getType,fileUrl))
                                        if r.status_code != 204:
                                            print("Error: 204=no file - have you updated the Preferences to switch on monitoring? See --perfs option")
                                        if debug:
                                            print("Returned:" + r.text)
                                        print("Continue to next SSP")
                                        continue
                                if save:
                                        file_name = path + "/SSP-filenames-" + SSPName + ".xml"
                                        file_object = open(file_name,"w")
                                        file_object.write(r.text)
                                        file_object.close()
                                # Parse the file of filenames
                                refResponse = r.text
                                refResponse = ET.fromstring(refResponse)
                                for reffeed in refResponse:
                                    if 'entry' in reffeed.tag:
                                        for entry in reffeed:
                                            if 'title' in entry.tag:
                                                filename = entry.text
                                            if 'link' in entry.tag:
                                                JSON_file_url = entry.get('href')
                                        if debug:
                                                print("JSON filename |%s|" % (filename))
                                                print("JSON URL |%s|" % (JSON_file_url))
                                        #            _         _        _
                                        #  __ _  ___| |_   ___| |_ __ _| |_ ___
                                        # / _` |/ _ \ __| / __| __/ _` | __/ __|
                                        #| (_| |  __/ |_  \__ \ || (_| | |_\__ \
                                        # \__, |\___|\__| |___/\__\__,_|\__|___/
                                        # |___/
                                        jsonHeaders = {'X-API-Session' : sessionToken }
                                        r = requests.get(JSON_file_url,headers=jsonHeaders,verify=False)
                                        if r.status_code != 200:
                                                print("Error: get SSP JSON file error code=%d filename=%s" %( r.status_code,filename))
                                                if debug:
                                                        print("Returned:" + r.text)
                                                sys.exit(r.status_code)
                                        if save:
                                                file_name = path + "/SSP-" + getType + "-" + SSPName + "-" + filename
                                                file_object = open(file_name,"w")
                                                file_object.write(r.text)
                                                file_object.close()

                                        # convert to JSON
                                        data = json.loads(r.text)

                                        if ( getType == "ProcessedMetrics" or getType == "AggregatedMetrics" ):
                                            sspName        = data["sspUtil"]["utilInfo"]["name"]
                                            clusterName    = data["sspUtil"]["utilInfo"]["clusterName"]
                                            frequency      = data["sspUtil"]["utilInfo"]["frequency"]
                                            startTimeStamp = data["sspUtil"]["utilInfo"]["startTimeStamp"]
                                            endTimeStamp   = data["sspUtil"]["utilInfo"]["endTimeStamp"]
                                            metricType     = data["sspUtil"]["utilInfo"]["metricType"]
                                        
                                            print("    SSP:%s cluster=%s frequency=%s type=%s" % \
                                                (sspName,clusterName,frequency,metricType ))
                                            print("    SSP:%s Start=%s" % (sspName,startTimeStamp))
                                            print("    SSP:%s End  =%s" % (sspName,endTimeStamp))

                                            if googlechart:
                                                file_name = "SSPGooglechart-" + getType + ".csv"
                                            else:
                                                file_name = "SSP-" + getType + ".csv"
                                            file_object = open(file_name,"a")

                                            if (vios):
                                                if googlechart:
                                                    vios_name = "SSPGooglechart-VIOS-" + getType + ".csv"
                                                else:
                                                    vios_name = "SSP-VIOS-" + getType + ".csv"
                                                vios_object = open(vios_name,"a")
                                      
                                            firsttime   = 1 # used to print only one header line per run
                                            samplecount = 0
                                            vioscount   = 0
                                            for sample in data["sspUtil"]["utilSamples"]:
                                                samplecount += 1
                                                timeStamp   = sample['sampleInfo']['timeStamp']
                                                free        = sample['poolUtil']['free'][0]
                                                size        = sample['poolUtil']['size'][0]
                                                numOfReads  = sample['poolUtil']['numOfReads'][0]
                                                numOfWrites = sample['poolUtil']['numOfWrites'][0]
                                                readBytes  = sample['poolUtil']['readBytes'][0]
                                                writeBytes = sample['poolUtil']['writeBytes'][0]
                                                readServiceTime = sample['poolUtil']['readServiceTime'][0]
                                                writeServiceTime = sample['poolUtil']['writeServiceTime'][0]

                                                if show:
                                                    print("%s size=%d free=%d OPs:read=%6d write=%6d Bytes:read=%9d write=%9d ServiceTime:read=%9d write=%9d" % (timeStamp,size,free,numOfReads,numOfWrites,readBytes,writeBytes,readServiceTime,writeServiceTime))
                                                if googlechart:
                                                    # Google chart Javascript Months start from 0 = January hence the -1
                                                    buffer = "%s, %s,%s, ,['Date(%s,%d,%s,%s,%s,%s)', %d,%d, %d,%d, %d,%d, %.2f,%.2f\n" % (getType,clusterName,sspName,timeStamp[0:4],int(timeStamp[5:7]) -1, timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19],size,free,numOfReads,numOfWrites,readBytes,writeBytes,float(readServiceTime)/1000000.0,float(writeServiceTime)/1000000.0)
                                                else:
                                                    buffer = "%s, %s,%s, %s,%s,%s, %s,%s,%s, %d,%d, %d,%d, %d,%d, %.2f,%.2f\n" % (getType,clusterName,sspName,timeStamp[0:4],timeStamp[5:7], timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19],size,free,numOfReads,numOfWrites,readBytes,writeBytes,float(readServiceTime)/1000000.0,float(writeServiceTime)/1000000.0)
                                                file_object.write(buffer)

                                                if vios:
                                                    if firsttime:
                                                        print("    SSP:%s Collecting VIOS data"%(sspName))
                                                    index=0
                                                    nname=[]
                                                    nrbytes=[]
                                                    nwbytes=[]
                                                    for node in sample['poolUtil']['nodeUtil']:
                                                        vioscount += 1
                                                        #nname = node['name']
                                                        nname.append(node['name'])
                                                        #nname[index] = nname[index].replace('.aixncc.uk.ibm.com', '')
                                                        npool = node['poolState'] # "UP or "DOWN"
                                                        nrbytes.append(node['readBytes'][0])
                                                        nwbytes.append(node['writeBytes'][0])
                                                        #if debug:
                                                            #nmtms = node['mtms'][0]
                                                            #nfree = node['free'][0]
                                                            #nsize = node['size'][0]
                                                            #print("name="+nname)
                                                            #print(" mtms="+nmtms)
                                                            #print(" pool="+npool)
                                                            #print("name="+nname+" mtms="+nmtms+" pool="+npool)
                                                            #print("nfree"+nfree+" nsize="+nsize+" rb="+nrbytes+" wb="+nwbytes)
                                                            #print("%s %s %s %d %d %d %d" % (nname,nmtms,npool,nfree,nsize,nrbytes,nwbytes))
                                                        if npool != "UP":
                                                            print("VIOS %s state is %s server is a %s" %(name,npool,nmtms))
                                                        index += 1
                                                    if firsttime == 1:
                                                        firsttime = 0
                                                        #print("%s, Header" %(clusterName), end='')
                                                        buffer = "%s, Header" %(clusterName)
                                                        for name in nname:
                                                            name = name.replace('.aixncc.uk.ibm.com', '')
                                                            #print(", %s-Read-KBs" % (name), end='')
                                                            buffer = buffer + ", '%s-Read-KBs'" % (name)
                                                        for name in nname:
                                                            name = name.replace('.aixncc.uk.ibm.com', '')
                                                            #print(", %s-Write-KBs" % (name), end='')
                                                            buffer = buffer + ", '%s-Write-KBs'" % (name)
                                                        #print("")
                                                        buffer = buffer + "\n"
                                                        if show:
                                                            print(buffer)
                                                        vios_object.write(buffer)
                                                    #print("%s, %s" %(timeStamp,clusterName), end='')
                                                    if googlechart:
                                                        # Google chart Javascript Months start from 0 = January hence the -1
                                                        buffer = "%s, ,['Date(%s,%d,%s,%s,%s,%s)'" %(clusterName,timeStamp[0:4],int(timeStamp[5:7]) -1, timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19])
                                                    else:
                                                        buffer = "%s, %s,%s,%s, %s,%s,%s" %(clusterName,timeStamp[0:4],timeStamp[5:7], timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19])
                                                    for bytes in nrbytes:
                                                        #print(", %d" % (bytes/1024), end='')
                                                        buffer = buffer + ", %d" % (bytes/1024)
                                                    for bytes in nwbytes:
                                                        #print(", -%d" % (bytes/1024), end='')
                                                        buffer = buffer + ", -%d" % (bytes/1024)
                                                    #print("")
                                                    if googlechart:
                                                        buffer = buffer + "]\n"
                                                    else:
                                                        buffer = buffer + "\n"
                                                    if show:
                                                        print(buffer)
                                                    vios_object.write(buffer)

                                                # end vios
                                            file_object.close()
                                            if vios:
                                               vios_object.close()
                                            print("    SSP:%s Saved %d samples and %d VIOS samples" %(SSPName,samplecount,vioscount))
                                            print("    SSP:%s %.2f%% Unused of %.2f GB" %(SSPName, float(free)/float(size)*100.0, float(size)/1024.0))
                                        else:
                                            #RawMetrices
                                            hasErrors = 0
                                            for errors in data["systemUtil"]["utilSample"]["errorInfo"]:
                                                # We only get here if it return from the GET with no error
                                                # but gave us a possibel array of error message in the file
                                                # Which sounds insane to me!!
                                                errnumber  = errors["errId"]
                                                errmessage = errors["errMsg"]
                                                hasErrors = 1
                                                print("Error: %s = %s" %(errnumber,errmessage))
                                                print("Skipping file:" + file_name+"\n")

                                            if ( hasErrors == 0 ):
                                                    timeStamp= data["systemUtil"]["utilSample"]["timeStamp"]
                                                    file_name = "SSP-" + getType + ".csv"
                                                    file_object = open(file_name,"a")
                                                    for sample in data["systemUtil"]["utilSample"]["sharedStoragePoolUtil"]:
                                                        file_object = open(file_name,"a")
                                                        sspName     = sample["name"]
                                                        clusterName = sample["clusterName"]
                                                        free        = sample['free']
                                                        size        = sample['size']
                                                        numOfReads  = sample['numOfReads']
                                                        numOfWrites = sample['numOfWrites']
                                                        readBytes   = sample['readBytes']
                                                        writeBytes  = sample['writeBytes']
                                                        readServiceTime = sample['readServiceTime']
                                                        writeServiceTime = sample['writeServiceTime']
                                                        if show:
                                                            print("%s %s size=%d free=%d OPs:reads=%6d writes=%6d Bytes:read=%9d write=%9d ServiceTime:read=%9d write=%9d" % (sspName,timeStamp,size,free,numOfReads,numOfWrites,readBytes,writeBytes,readServiceTime,writeServiceTime))
                                                        #buffer = "Type, Cluster,SSP, YY,MM,DD,HH,MM,SS, size,free, numOfReads,numOfWrites, readBytes,writeBytes, readServiceTime,writeServiceTime\n"
                                                        #file_object.write(buffer)
                                                        buffer = "%s, %s,%s, %s,%s,%s,%s,%s,%s, %d,%d, %d,%d, %d,%d, %d,%d]\n" % (getType,clusterName,sspName,timeStamp[0:4],timeStamp[5:7], timeStamp[8:10], timeStamp[11:13], timeStamp[14:16],timeStamp[17:19],size,free,numOfReads,numOfWrites,readBytes,writeBytes,readServiceTime,writeServiceTime)
                                                        file_object.write(buffer)
                                                    file_object.close()
                            else:
                                print("        Ignoring the SSP")
sys.exit("Successful")
